<?php

class WP3CXW_Editor {

	private $webinar_form;
	private $panels = array();

	public function __construct( WP3CXW_WebinarForm $webinar_form ) {
		$this->webinar_form = $webinar_form;
	}

	public function add_panel( $id, $title, $callback ) {
		if ( wp3cxw_is_name( $id ) ) {
			$this->panels[$id] = array(
				'title' => $title,
				'callback' => $callback,
			);
		}
	}

	public function display() {
		if ( empty( $this->panels ) ) {
			return;
		}

		echo '<ul id="webinar-form-editor-tabs">';

		foreach ( $this->panels as $id => $panel ) {
			echo sprintf( '<li id="%1$s-tab"><a href="#%1$s">%2$s</a></li>',
				esc_attr( $id ), esc_html( $panel['title'] ) );
		}

		echo '</ul>';

		foreach ( $this->panels as $id => $panel ) {
			echo sprintf( '<div class="webinar-form-editor-panel" id="%1$s">',
				esc_attr( $id ) );

			if ( is_callable( $panel['callback'] ) ) {
				$this->notice( $id, $panel );
				call_user_func( $panel['callback'], $this->webinar_form );
			}

			echo '</div>';
		}
	}

	public function notice( $id, $panel ) {
		echo '<div class="config-error"></div>';
	}
}

function wp3cxw_editor_panel_config( $post ) {
	wp3cxw_editor_box_config( $post );
}

function wp3cxw_editor_box_config( $post, $args = '' ) {
	$args = wp_parse_args( $args, array(
		'id' => 'wp3cxw-config',
		'name' => 'config',
		'title' => __( 'Configuration', '3cx-webinar' ),
		'use' => null,
	) );

	$id = esc_attr( $args['id'] );

	$config = wp_parse_args( $post->prop( $args['name'] ), array(
		'active' => false,
		'apitoken' => '',
		'portalfqdn' => '',
		'extension' => '',
		'country' => '',
		'maxparticipants' => 0
	) );

?>
<div class="webinar-form-editor-box-config" id="<?php echo $id; ?>">
<h2><?php echo esc_html( $args['title'] ); ?></h2>

<fieldset>
<legend>
<?php
	$desc_link = wp3cxw_link(
		__( 'https://www.3cx.com/setting-up-mail/', '3cx-webinar' ),
		__( 'Setting Up Mail', '3cx-webinar' ) );
?>
</legend>
<table class="form-table">
<tbody>
	<tr>
	<th scope="row">
		<label for="<?php echo $id; ?>-portalfqdn"><?php echo esc_html( __( 'WebMeeting FQDN', '3cx-webinar' ) ); ?>&nbsp;
		<img src="<?php echo wp3cxw_plugin_url('images/info.png');?>" alt="info" title="<?php echo esc_html(__('This is your WebMeeting Portal FQDN. Look at PBX Management Console, WebMeeting section. It usually ends in 3cx.net','3cx-webinar'));?>"/></label>
	</th>
	<td>
		<input type="text" id="<?php echo $id; ?>-portalfqdn" name="<?php echo $id; ?>[portalfqdn]" class="large-text code" size="70" value="<?php echo esc_attr( $config['portalfqdn'] ); ?>" data-config-field="<?php echo sprintf( '%s.portalfqdn', esc_attr( $args['name'] ) ); ?>" />
	</td>
	</tr>
	
	<tr>
	<th scope="row">
		<label for="<?php echo $id; ?>-apitoken"><?php echo esc_html( __( 'WebMeeting API Token', '3cx-webinar' ) ); ?>
		<img src="<?php echo wp3cxw_plugin_url('images/info.png');?>" alt="info" title="<?php echo esc_html(__('This is you WebMeeting API Token, it is assigned by PBX. Look at PBX Management Console, WebMeeting section, must be a 32 chars string','3cx-webinar'));?>"/></label>
	</th>
	<td>
		<input type="text" id="<?php echo $id; ?>-apitoken" name="<?php echo $id; ?>[apitoken]" class="large-text code" size="70" value="<?php echo esc_attr( $config['apitoken'] ); ?>" data-config-field="<?php echo sprintf( '%s.apitoken', esc_attr( $args['name'] ) ); ?>" />
	</td>
	</tr>	
	
	<tr>
	<th scope="row">
		<label for="<?php echo $id; ?>-extension"><?php echo esc_html( __( '3CX Extension Number', '3cx-webinar' ) ); ?>
		<img src="<?php echo wp3cxw_plugin_url('images/info.png');?>" alt="info" title="<?php echo esc_html(__('This is extension phone number associated with your webinar list. You can add different forms for different extensions.','3cx-webinar'));?>"/></label>
	</th>
	<td>
		<input type="text" id="<?php echo $id; ?>-extension" name="<?php echo $id; ?>[extension]" class="large-text code" size="70" value="<?php echo esc_attr( $config['extension'] ); ?>" data-config-field="<?php echo sprintf( '%s.extension', esc_attr( $args['name'] ) ); ?>" />
	</td>
	</tr>		
	
	<tr>
	<th scope="row">
		<label for="<?php echo $id; ?>-country"><?php echo esc_html( __( 'Webinar country/language', '3cx-webinar' ) ); ?>
		<img src="<?php echo wp3cxw_plugin_url('images/info.png');?>" alt="info" title="<?php echo esc_html(__('This a country you can display together with your webinar rows.','3cx-webinar'));?>"/></label>
	</th>
	<td>
		<select id="<?php echo $id; ?>-country" name="<?php echo $id; ?>[country]" class="large-text code" style="width:500px" data-config-field="<?php echo sprintf( '%s.country', esc_attr( $args['name'] ) ); ?>">
      <option value='' data-image="<?php echo wp3cxw_plugin_url('images/blank.png');?>">None</option>
		<?php
			foreach(wp3cxw_get_countries() as $code=>$name){?>
			
			<option value="<?php echo $code;?>" <?php if ($code==$config['country']) echo 'selected ';?> data-imagecss="flags <?php echo $code;?>" data-image="<?php echo wp3cxw_plugin_url('images/blank.png');?>" data-title="<?php echo $name;?>"/><?php echo $name;?></option>
		<?php
			} 
		?>	
    </select>		
	</td>
	</tr>			
	
	<tr>
	<th scope="row">
		<label for="<?php echo $id; ?>-maxparticipants"><?php echo esc_html( __( 'Limit subscribers to', '3cx-webinar' ) ); ?>
		<img src="<?php echo wp3cxw_plugin_url('images/info.png');?>" alt="info" title="<?php echo esc_html(__('You can limit the number of subscribers for this webinar. 0 means unrestricted (3CX WebMeeting license is always enforced by server)','3cx-webinar'));?>"/></label>
	</th>
	<td>
		<input type="text" id="<?php echo $id; ?>-maxparticipants" name="<?php echo $id; ?>[maxparticipants]" class="large-text code" size="70" value="<?php echo esc_attr( $config['maxparticipants'] ); ?>" data-config-field="<?php echo sprintf( '%s.maxparticipants', esc_attr( $args['name'] ) ); ?>" />
	</td>
	</tr>			
</tbody>
</table>
</fieldset>
</div>
<?php
}

