<?php

add_action( 'parse_request', 'wp3cxw_control_init', 20 );

function wp3cxw_control_init() {
	if ( isset( $_POST['_wp3cxw'] ) ) {
		$webinar_form = wp3cxw_webinar_form(intval($_POST['_wp3cxw']));

		if ( $webinar_form ) {
			$webinar_form->submit();
		}
	}
}

add_filter( 'widget_text', 'wp3cxw_widget_text_filter', 9 );

function wp3cxw_widget_text_filter( $content ) {
	$pattern = '/\[[\r\n\t ]*webinar-form?[\r\n\t ].*?\]/';

	if ( ! preg_match( $pattern, $content ) ) {
		return $content;
	}

	$content = do_shortcode( $content );

	return $content;
}

add_action( 'wp_enqueue_scripts', 'wp3cxw_do_enqueue_scripts' );

function wp3cxw_do_enqueue_scripts() {
	if ( wp3cxw_load_js() ) {
		wp3cxw_enqueue_scripts();
	}

	if ( wp3cxw_load_css() ) {
		wp3cxw_enqueue_styles();
	}
}

function wp3cxw_enqueue_scripts() {
	$in_footer = true;

	if ( 'header' === wp3cxw_load_js() ) {
		$in_footer = false;
	}
	wp_enqueue_style('3cx-webinar-jquery-ui',wp3cxw_plugin_url( 'includes/css/jquery-ui.css'), null, WP3CXW_VERSION);
	wp_enqueue_style('3cx-webinar-flags',wp3cxw_plugin_url( 'includes/css/flags.css'), null, WP3CXW_VERSION);

	wp_enqueue_script('jquery-ui-dialog');

	wp_enqueue_script('3cx-webinar', wp3cxw_plugin_url( 'includes/js/scripts.js' ), array( 'jquery' ), WP3CXW_VERSION, $in_footer );
	wp_enqueue_script('3cx-webinar-jsrender', wp3cxw_plugin_url( 'includes/js/jsrender.min.js'), null, WP3CXW_VERSION);
	wp_enqueue_script('3cx-webinar-jstz', wp3cxw_plugin_url( 'includes/js/jstz.min.js' ), null, WP3CXW_VERSION);
	wp_enqueue_script('3cx-webinar-jsdd', wp3cxw_plugin_url( 'includes/js/jquery.dd.js' ), null, WP3CXW_VERSION);

	$wp3cxw = array(
		'apiSettings' => array(
			'root' => esc_url_raw( rest_url( '3cx-webinar/v1' ) ),
			'namespace' => '3cx-webinar/v1',
		),
	);

	if ( defined( 'WP_CACHE' ) && WP_CACHE ) {
		$wp3cxw['cached'] = 1;
	}

	if ( wp3cxw_support_html5_fallback() ) {
		$wp3cxw['jqueryUi'] = 1;
	}

	wp_localize_script( '3cx-webinar', 'wp3cxw', $wp3cxw );

	do_action( 'wp3cxw_enqueue_scripts' );
}

function wp3cxw_script_is() {
	return wp_script_is( '3cx-webinar' );
}

function wp3cxw_enqueue_styles() {
	wp_enqueue_style( '3cx-webinar',
		wp3cxw_plugin_url( 'includes/css/styles.css' ),
		array(), WP3CXW_VERSION, 'all' );

	if ( wp3cxw_is_rtl() ) {
		wp_enqueue_style( '3cx-webinar-rtl',
			wp3cxw_plugin_url( 'includes/css/styles-rtl.css' ),
			array(), WP3CXW_VERSION, 'all' );
	}

	do_action( 'wp3cxw_enqueue_styles' );
}

function wp3cxw_style_is() {
	return wp_style_is( '3cx-webinar' );
}

/* HTML5 Fallback */

add_action( 'wp_enqueue_scripts', 'wp3cxw_html5_fallback', 20 );

function wp3cxw_html5_fallback() {
	if ( ! wp3cxw_support_html5_fallback() ) {
		return;
	}

	if ( wp3cxw_script_is() ) {
		wp_enqueue_script( 'jquery-ui-datepicker' );
		wp_enqueue_script( 'jquery-ui-spinner' );
	}

	if ( wp3cxw_style_is() ) {
		wp_enqueue_style( 'jquery-ui-smoothness',
			wp3cxw_plugin_url(
				'includes/js/jquery-ui/themes/smoothness/jquery-ui.min.css' ),
			array(), '1.11.4', 'screen' );
	}
}
